// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go_gapic. DO NOT EDIT.

package binaryauthorization

import (
	"context"
	"fmt"
	"math"
	"net/url"

	gax "github.com/googleapis/gax-go/v2"
	"google.golang.org/api/option"
	"google.golang.org/api/option/internaloption"
	gtransport "google.golang.org/api/transport/grpc"
	binaryauthorizationpb "google.golang.org/genproto/googleapis/cloud/binaryauthorization/v1"
	"google.golang.org/grpc"
	"google.golang.org/grpc/metadata"
)

var newSystemPolicyClientHook clientHook

// SystemPolicyCallOptions contains the retry settings for each method of SystemPolicyClient.
type SystemPolicyCallOptions struct {
	GetSystemPolicy []gax.CallOption
}

func defaultSystemPolicyGRPCClientOptions() []option.ClientOption {
	return []option.ClientOption{
		internaloption.WithDefaultEndpoint("binaryauthorization.googleapis.com:443"),
		internaloption.WithDefaultMTLSEndpoint("binaryauthorization.mtls.googleapis.com:443"),
		internaloption.WithDefaultAudience("https://binaryauthorization.googleapis.com/"),
		internaloption.WithDefaultScopes(DefaultAuthScopes()...),
		internaloption.EnableJwtWithScope(),
		option.WithGRPCDialOption(grpc.WithDefaultCallOptions(
			grpc.MaxCallRecvMsgSize(math.MaxInt32))),
	}
}

func defaultSystemPolicyCallOptions() *SystemPolicyCallOptions {
	return &SystemPolicyCallOptions{
		GetSystemPolicy: []gax.CallOption{},
	}
}

// internalSystemPolicyClient is an interface that defines the methods available from Binary Authorization API.
type internalSystemPolicyClient interface {
	Close() error
	setGoogleClientInfo(...string)
	Connection() *grpc.ClientConn
	GetSystemPolicy(context.Context, *binaryauthorizationpb.GetSystemPolicyRequest, ...gax.CallOption) (*binaryauthorizationpb.Policy, error)
}

// SystemPolicyClient is a client for interacting with Binary Authorization API.
// Methods, except Close, may be called concurrently. However, fields must not be modified concurrently with method calls.
//
// API for working with the system policy.
type SystemPolicyClient struct {
	// The internal transport-dependent client.
	internalClient internalSystemPolicyClient

	// The call options for this service.
	CallOptions *SystemPolicyCallOptions
}

// Wrapper methods routed to the internal client.

// Close closes the connection to the API service. The user should invoke this when
// the client is no longer required.
func (c *SystemPolicyClient) Close() error {
	return c.internalClient.Close()
}

// setGoogleClientInfo sets the name and version of the application in
// the `x-goog-api-client` header passed on each request. Intended for
// use by Google-written clients.
func (c *SystemPolicyClient) setGoogleClientInfo(keyval ...string) {
	c.internalClient.setGoogleClientInfo(keyval...)
}

// Connection returns a connection to the API service.
//
// Deprecated.
func (c *SystemPolicyClient) Connection() *grpc.ClientConn {
	return c.internalClient.Connection()
}

// GetSystemPolicy gets the current system policy in the specified location.
func (c *SystemPolicyClient) GetSystemPolicy(ctx context.Context, req *binaryauthorizationpb.GetSystemPolicyRequest, opts ...gax.CallOption) (*binaryauthorizationpb.Policy, error) {
	return c.internalClient.GetSystemPolicy(ctx, req, opts...)
}

// systemPolicyGRPCClient is a client for interacting with Binary Authorization API over gRPC transport.
//
// Methods, except Close, may be called concurrently. However, fields must not be modified concurrently with method calls.
type systemPolicyGRPCClient struct {
	// Connection pool of gRPC connections to the service.
	connPool gtransport.ConnPool

	// flag to opt out of default deadlines via GOOGLE_API_GO_EXPERIMENTAL_DISABLE_DEFAULT_DEADLINE
	disableDeadlines bool

	// Points back to the CallOptions field of the containing SystemPolicyClient
	CallOptions **SystemPolicyCallOptions

	// The gRPC API client.
	systemPolicyClient binaryauthorizationpb.SystemPolicyV1Client

	// The x-goog-* metadata to be sent with each request.
	xGoogMetadata metadata.MD
}

// NewSystemPolicyClient creates a new system policy v1 client based on gRPC.
// The returned client must be Closed when it is done being used to clean up its underlying connections.
//
// API for working with the system policy.
func NewSystemPolicyClient(ctx context.Context, opts ...option.ClientOption) (*SystemPolicyClient, error) {
	clientOpts := defaultSystemPolicyGRPCClientOptions()
	if newSystemPolicyClientHook != nil {
		hookOpts, err := newSystemPolicyClientHook(ctx, clientHookParams{})
		if err != nil {
			return nil, err
		}
		clientOpts = append(clientOpts, hookOpts...)
	}

	disableDeadlines, err := checkDisableDeadlines()
	if err != nil {
		return nil, err
	}

	connPool, err := gtransport.DialPool(ctx, append(clientOpts, opts...)...)
	if err != nil {
		return nil, err
	}
	client := SystemPolicyClient{CallOptions: defaultSystemPolicyCallOptions()}

	c := &systemPolicyGRPCClient{
		connPool:           connPool,
		disableDeadlines:   disableDeadlines,
		systemPolicyClient: binaryauthorizationpb.NewSystemPolicyV1Client(connPool),
		CallOptions:        &client.CallOptions,
	}
	c.setGoogleClientInfo()

	client.internalClient = c

	return &client, nil
}

// Connection returns a connection to the API service.
//
// Deprecated.
func (c *systemPolicyGRPCClient) Connection() *grpc.ClientConn {
	return c.connPool.Conn()
}

// setGoogleClientInfo sets the name and version of the application in
// the `x-goog-api-client` header passed on each request. Intended for
// use by Google-written clients.
func (c *systemPolicyGRPCClient) setGoogleClientInfo(keyval ...string) {
	kv := append([]string{"gl-go", versionGo()}, keyval...)
	kv = append(kv, "gapic", getVersionClient(), "gax", gax.Version, "grpc", grpc.Version)
	c.xGoogMetadata = metadata.Pairs("x-goog-api-client", gax.XGoogHeader(kv...))
}

// Close closes the connection to the API service. The user should invoke this when
// the client is no longer required.
func (c *systemPolicyGRPCClient) Close() error {
	return c.connPool.Close()
}

func (c *systemPolicyGRPCClient) GetSystemPolicy(ctx context.Context, req *binaryauthorizationpb.GetSystemPolicyRequest, opts ...gax.CallOption) (*binaryauthorizationpb.Policy, error) {
	md := metadata.Pairs("x-goog-request-params", fmt.Sprintf("%s=%v", "name", url.QueryEscape(req.GetName())))

	ctx = insertMetadata(ctx, c.xGoogMetadata, md)
	opts = append((*c.CallOptions).GetSystemPolicy[0:len((*c.CallOptions).GetSystemPolicy):len((*c.CallOptions).GetSystemPolicy)], opts...)
	var resp *binaryauthorizationpb.Policy
	err := gax.Invoke(ctx, func(ctx context.Context, settings gax.CallSettings) error {
		var err error
		resp, err = c.systemPolicyClient.GetSystemPolicy(ctx, req, settings.GRPC...)
		return err
	}, opts...)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
